<?php

namespace PostTypeBuilder;

use \ReflectionProperty;

class PostMetaManager{
	private $keys = array();
	private $single_keys = array();
	private $array_keys = array();
	private $array_key_aliases = array();
	
	private $properties;
	
	private $load_query;
	private $remove_query;
	private $add_query;
	
	private $registered_property_types = null;
	
	function __construct($qualified_class_name){
		global $wpdb;
		
		$class_meta = PostTypeBuilder::$registered_class_names[$qualified_class_name];
		
		$qualified_class_name = $class_meta->qualified_class_name;
		
		$instance = new $qualified_class_name();
		
		foreach($class_meta->class_reflector->getProperties(ReflectionProperty::IS_PUBLIC) as $property){
			if(!$property->hasAnnotation("Addendum\\Property")){
				continue;
			}
			
			$key = $property->getName();
			
			$this->keys[] = $key;
			
			if(is_array($instance->$key)){
				$this->array_keys[] = $key;
				
				$key_has_alias = false;
				
				if($key_has_alias){
					
				} else {
					$key_alias = $key . "s";
				}
				
				$this->array_key_aliases[$key_alias] = $key;
			} else {
				$this->single_keys[] = $key;
			}
		}
		
		//
		
		$this->registered_property_types = array();
		
		foreach($class_meta->class_reflector->getProperties(ReflectionProperty::IS_PUBLIC) as $property){
			if(!$property->hasAnnotation("Addendum\\Property")){
				continue;
			}
			
			$property_annotation = $property->getAnnotation("Addendum\\Property");
			
			$this->registered_property_types[$property->getName()] = $property_annotation->type;
		}
		
		//
		
		if(count($this->keys) > 0){
			$this->load_query = "SELECT meta_key, meta_value FROM {$wpdb->postmeta} WHERE post_id = %d AND (meta_key = '" . implode("' OR meta_key = '", $this->keys) . "')";
			$this->remove_query = "DELETE FROM {$wpdb->postmeta} WHERE post_id = %d AND (meta_key = '" . implode("' OR meta_key = '", $this->keys) . "')";
			
			$add_values = array();
			
			foreach($this->single_keys as $key){
				$add_values[] = "(%d,'{$key}',%s)";
			}
			
			$this->add_query = "INSERT INTO {$wpdb->postmeta} (post_id,meta_key,meta_value) VALUES " . implode(",", $add_values) . "";
		}
	}
	
	public function save($instance){
		$this->remove($instance);
		$this->add($instance);
	}
	
	public function load($instance){
		if(count($this->keys) > 0){
			global $wpdb;
			
			$query = $wpdb->prepare($this->load_query, $instance->ID);
			
			foreach($wpdb->get_results($query) as $result){
				$key = $result->meta_key;
				
				if(!in_array($key, $this->keys)){
					continue;
				}
				
				$value = $result->meta_value;
				
				if(is_array($instance->$key)){
					$array = &$instance->$key;
					$array[] = $value;
				} else {
					$instance->$key = $value;
				}
			}
		}
	}
	
	private function remove($instance){
		if(count($this->keys) > 0){
			global $wpdb;
			
			$query = $wpdb->prepare($this->remove_query, $instance->ID);
			
			$wpdb->query($query);
		}
	}
	
	private function add($instance){
		$this->add_single_keys($instance);
		$this->add_array_keys($instance);
	}
	
	private function add_single_keys($instance){
		if(count($this->single_keys) == 0){
			return;
		}
		
		global $wpdb;
		
		$parameters = array();
		
		$instance_id = $instance->ID;
		
		foreach($this->single_keys as $key){
			$parameters[] = $instance_id;
			$parameters[] = $instance->$key;
		}
		
		$query = $wpdb->prepare($this->add_query, $parameters);
		$wpdb->query($query);
	}
	
	private function add_array_keys($instance){
		foreach($this->array_keys as $key){
			$this->add_array_key($instance, $key);
		}
	}
	
	private function add_array_key($instance, $key){
		$values = $instance->$key;
		
		if(count($values) == 0){
			return;
		}
		
		global $wpdb;
		
		$add_values = array();
		
		foreach($values as $value){
			$add_values[] = "(%d,'{$key}',%s)";
		}
		
		$add_query = "INSERT INTO {$wpdb->postmeta} (post_id,meta_key,meta_value) VALUES " . implode(",", $add_values) . "";
		
		$instance_id = $instance->ID;
		
		$parameters = array();
		
		for($i = count($values) - 1; $i >= 0; $i--){
			$value = $values[$i];
			$parameters[] = $instance_id;
			$parameters[] = $value;
		}
		
		$query = $wpdb->prepare($add_query, $parameters);
		$wpdb->query($query);
	}
	
	public function lazy_load_single($instance, $property_name){
		$type_name = $this->registered_property_types[$property_name];
			$id = $instance->$property_name;
			
		
		if($type_name == "Image"){
			return get_post($id);
		}
		
		if($type_name == "User"){
			return get_userdata($id);
		}
		
		if(array_key_exists($type_name, PostTypeBuilder::$registered_class_names)){
			$class_meta = PostTypeBuilder::$registered_class_names[$type_name];
			
			$qualified_class_name = $class_meta->qualified_class_name;
			
			return new $qualified_class_name($id);
		}
		
		return null;
	}
	
	public function lazy_load_multiple($instance, $property_name){
		$type_name = $this->registered_property_types[$property_name];
		
		$ids = $instance->$property_name;
		
		if($type_name == "Image"){
			return get_posts(array("post__in" => $ids, "post_type" => "attachment", "post_status" => "any")); // working ?
		}
		
		if($type_name == "User"){
			$users = array();
			
			foreach($ids as $id){
				$users[] = get_userdata($id);
			}
			
			return $users;
		}
		
		if(array_key_exists($type_name, PostTypeBuilder::$registered_class_names)){
			$class_meta = PostTypeBuilder::$registered_class_names[$type_name];
			
			$qualified_class_name = $class_meta->qualified_class_name;
			
			$instances = array();
			
			foreach($ids as $id){
				$instances[] = new $qualified_class_name($id);
			}
			
			return $instances;
		}
		
		return null;
	}
	
	public function has_property($property_name){
		return array_key_exists($property_name, $this->registered_property_types);
	}
}